@@ -1,4 +1,4 @@
-/* $Id: dialogs.c 28379 2008-10-23 09:37:07Z stephan $ */
+/* $Id: dialogs.c 28003 2008-10-01 07:57:03Z kelnos $ */
 /*
  * Copyright (c) 2003,2005 Jasper Huijsmans <jasper@xfce.org>
  * Copyright (c) 2003 Benedikt Meurer <benny@xfce.org>
@@ -60,7 +60,15 @@
 /* max. length of a message */
 #define MAXMESSAGELEN   2048
 
+/* Darwin does not have a variable named environ
+   but has a function which you can get the environ
+   variable with.  */
+#if defined(__APPLE__) && defined(__MACH__)
+#include <crt_externs.h>
+#define environ (*_NSGetEnviron())
+#else
 extern char **environ;
+#endif
 
 /* create a header with optional icon (may be NULL) in larger bold font;
  * background and foreground colors are taken from gtk */
@@ -267,7 +275,7 @@ message_dialog (GtkMessageType type, const gchar * message)
 
     dlg = gtk_message_dialog_new (NULL,
 				  GTK_DIALOG_MODAL,
-				  type, GTK_BUTTONS_CLOSE, "%s", message);
+				  type, GTK_BUTTONS_CLOSE, message);
 
     xfce_gtk_window_center_on_monitor_with_pointer (GTK_WINDOW (dlg));
     gtk_dialog_run (GTK_DIALOG (dlg));
@@ -436,7 +444,7 @@ xfce_confirm (const gchar *text,
     dialog = gtk_message_dialog_new (NULL,
                                      GTK_DIALOG_MODAL,
                                      GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
-                                     "%s", text);
+                                     text);
     if (strcmp (stock_id, GTK_STOCK_YES) == 0)
         button = gtk_button_new_from_stock (GTK_STOCK_NO);
     else
