@@ -43,6 +43,8 @@
 #define FALLBACK_NAME "??"
 #define ALL_WORKSPACES (0xFFFFFFFF)
 
+#define NETK_WINDOW_GET_PRIVATE(o)    (G_TYPE_INSTANCE_GET_PRIVATE ((o), NETK_TYPE_WINDOW, NetkWindowPrivate))
+
 static GHashTable *window_hash = NULL;
 
 /* Keep 0-10 in sync with the numbers in the WindowState enum. Yeah I'm
@@ -69,6 +71,7 @@ struct _NetkWindowPrivate
     NetkClassGroup *class_group;
     Window group_leader;
     Window transient_for;
+    GdkRectangle icon_geometry;
     char *name;
     char *icon_name;
     char *session_id;
@@ -132,6 +135,7 @@ struct _NetkWindowPrivate
     guint need_update_transient_for:1;
     guint need_update_wmclass : 1;
     guint need_update_wmhints : 1;
+    guint need_update_icon_geometry : 1;
 
     /* if available */
     char *client_machine;
@@ -178,48 +182,39 @@ static void force_update_now (NetkWindow * window);
 
 static NetkWindow *find_last_transient_for (GList * windows, Window xwindow);
 
-static gpointer parent_class;
+static GObjectClass *parent_class;
 static guint signals[LAST_SIGNAL] = { 0 };
 
 GType
 netk_window_get_type (void)
 {
-    static GType object_type = 0;
+    static GType type = G_TYPE_INVALID;
 
-    g_type_init ();
-
-    if (!object_type)
+    if (G_UNLIKELY (type == G_TYPE_INVALID))
     {
-        static const GTypeInfo object_info = {
-            sizeof (NetkWindowClass),
-            (GBaseInitFunc) NULL,
-            (GBaseFinalizeFunc) NULL,
-            (GClassInitFunc) netk_window_class_init,
-            NULL,               /* class_finalize */
-            NULL,               /* class_data */
-            sizeof (NetkWindow),
-            0,                  /* n_preallocs */
-            (GInstanceInitFunc) netk_window_init,
-            NULL                /* value_table */
-        };
-
-        object_type =
-            g_type_register_static (G_TYPE_OBJECT, "NetkWindow", &object_info,
-                                    0);
+        type = _netk_g_type_register_simple (G_TYPE_OBJECT,
+                                             "NetkWindow",
+                                             sizeof (NetkWindowClass),
+                                             netk_window_class_init,
+                                             sizeof (NetkWindow),
+                                             netk_window_init,
+                                             0);
     }
 
-    return object_type;
+    return type;
 }
 
 static void
 netk_window_init (NetkWindow * window)
 {
-    window->priv = g_new0 (NetkWindowPrivate, 1);
-
+    window->priv = NETK_WINDOW_GET_PRIVATE (window);
+    
     window->priv->name = g_strdup (FALLBACK_NAME);;
     window->priv->icon_name = NULL;
     window->priv->client_machine = NULL;
     window->priv->workspace = ALL_WORKSPACES;
+    
+    window->priv->icon_geometry.width = -1;
 
     window->priv->icon_cache = p_netk_icon_cache_new ();
 }
@@ -228,19 +223,21 @@ static void
 netk_window_class_init (NetkWindowClass * klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
+    
+    g_type_class_add_private (klass, sizeof (NetkWindowPrivate));
 
     parent_class = g_type_class_peek_parent (klass);
 
     object_class->finalize = netk_window_finalize;
 
     signals[NAME_CHANGED] =
-        g_signal_new ("name_changed", G_OBJECT_CLASS_TYPE (object_class),
+        g_signal_new (I_("name_changed"), G_OBJECT_CLASS_TYPE (object_class),
                       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (NetkWindowClass,
                                                           name_changed), NULL,
                       NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
     signals[STATE_CHANGED] =
-        g_signal_new ("state_changed", G_OBJECT_CLASS_TYPE (object_class),
+        g_signal_new (I_("state_changed"), G_OBJECT_CLASS_TYPE (object_class),
                       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (NetkWindowClass,
                                                           state_changed),
                       NULL, NULL, p_netk_marshal_VOID__FLAGS_FLAGS,
@@ -248,19 +245,19 @@ netk_window_class_init (NetkWindowClass * klass)
                       NETK_TYPE_WINDOW_STATE);
 
     signals[WORKSPACE_CHANGED] =
-        g_signal_new ("workspace_changed", G_OBJECT_CLASS_TYPE (object_class),
+        g_signal_new (I_("workspace_changed"), G_OBJECT_CLASS_TYPE (object_class),
                       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (NetkWindowClass,
                                                           workspace_changed),
                       NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
     signals[ICON_CHANGED] =
-        g_signal_new ("icon_changed", G_OBJECT_CLASS_TYPE (object_class),
+        g_signal_new (I_("icon_changed"), G_OBJECT_CLASS_TYPE (object_class),
                       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (NetkWindowClass,
                                                           icon_changed), NULL,
                       NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
     signals[ACTIONS_CHANGED] =
-        g_signal_new ("actions_changed", G_OBJECT_CLASS_TYPE (object_class),
+        g_signal_new (I_("actions_changed"), G_OBJECT_CLASS_TYPE (object_class),
                       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (NetkWindowClass,
                                                           actions_changed),
                       NULL, NULL, p_netk_marshal_VOID__FLAGS_FLAGS,
@@ -268,7 +265,7 @@ netk_window_class_init (NetkWindowClass * klass)
                       NETK_TYPE_WINDOW_ACTIONS);
 
     signals[GEOMETRY_CHANGED] =
-        g_signal_new ("geometry_changed", G_OBJECT_CLASS_TYPE (object_class),
+        g_signal_new (I_("geometry_changed"), G_OBJECT_CLASS_TYPE (object_class),
                       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (NetkWindowClass,
                                                           geometry_changed),
                       NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE,
@@ -302,8 +299,6 @@ netk_window_finalize (GObject * object)
     if (window->priv->app)
         g_object_unref (G_OBJECT (window->priv->app));
 
-    g_free (window->priv);
-
     G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -390,6 +385,7 @@ p_netk_window_create (Window xwindow, NetkScreen * screen)
     window->priv->need_update_transient_for = TRUE;
     window->priv->need_update_wmclass = TRUE;
     window->priv->need_update_wmhints = TRUE;
+    window->priv->need_update_icon_geometry = TRUE;
     force_update_now (window);
 
     return window;
@@ -1278,6 +1274,42 @@ netk_window_is_visible_on_workspace (NetkWindow * window,
     return netk_window_is_on_workspace (window, workspace);
 }
 
+/**
+ * netk_window_get_icon_geometry:
+ * @window: a #NetkWindow
+ * @xp: return location for X coordinate of icon
+ * @yp: return location for Y coordinate of icon
+ * @widthp: return location for width of icon
+ * @heightp: return location for height of icon
+ *
+ * Gets the size and position of the icon.
+ * returns %TRUE if there was one.
+ * 
+ **/
+gboolean
+netk_window_get_icon_geometry (NetkWindow * window, int *xp, int *yp, int *widthp, int *heightp)
+{
+    g_return_val_if_fail (NETK_IS_WINDOW (window), FALSE);
+    
+    if (window->priv->icon_geometry.width == -1) {
+        return FALSE;
+    }
+
+    if (xp != NULL)
+        *xp = window->priv->icon_geometry.x;
+
+    if (yp != NULL)
+        *yp = window->priv->icon_geometry.y;
+
+    if (widthp != NULL)
+        *widthp = window->priv->icon_geometry.width;
+
+    if (heightp != NULL)
+        *heightp = window->priv->icon_geometry.height;
+        
+    return TRUE;
+}
+
 /**
  * netk_window_set_icon_geometry:
  * @window: a #NetkWindow
@@ -1292,6 +1324,17 @@ void
 netk_window_set_icon_geometry (NetkWindow * window, int x, int y, int width,
                                int height)
 {
+    if ((window->priv->icon_geometry.x == x)  &&
+        (window->priv->icon_geometry.y == y) &&
+        (window->priv->icon_geometry.width == width) &&
+        (window->priv->icon_geometry.height == height))
+            return;
+
+    window->priv->icon_geometry.x = x;
+    window->priv->icon_geometry.y = y;
+    window->priv->icon_geometry.width = width;
+    window->priv->icon_geometry.height = height;
+
     p_netk_set_icon_geometry (window->priv->xwindow, x, y, width, height);
 }
 
@@ -1373,8 +1416,10 @@ p_netk_window_set_class_group (NetkWindow     *window,
 
     if (class_group)
         g_object_ref (G_OBJECT (class_group));
+
     if (window->priv->class_group)
         g_object_unref (G_OBJECT (window->priv->class_group));
+
     window->priv->class_group = class_group;
 }
 
@@ -1445,6 +1490,11 @@ p_netk_window_process_property_notify (NetkWindow * window, XEvent * xevent)
         window->priv->need_update_wmhints = TRUE;
         queue_update (window);
     }
+    else if (xevent->xproperty.atom == p_netk_atom_get ("_NET_WM_ICON_GEOMETRY"))
+    {
+        window->priv->need_update_icon_geometry = TRUE;
+        queue_update (window);
+    }
 }
 
 void
@@ -1875,6 +1925,18 @@ update_wmclass (NetkWindow *window)
                         &window->priv->res_name);
 }
 
+static void 
+update_icon_geometry (NetkWindow *window)
+{
+    if (!window->priv->need_update_icon_geometry)
+        return;
+        
+    window->priv->need_update_icon_geometry = FALSE;
+    if (!p_netk_get_icon_geometry(window->priv->xwindow, &window->priv->icon_geometry.x, &window->priv->icon_geometry.y, &window->priv->icon_geometry.width, &window->priv->icon_geometry.height)) {
+        window->priv->icon_geometry.width = -1;
+    }
+}
+
 static void
 update_wmhints (NetkWindow *window)
 {
@@ -1945,13 +2007,13 @@ force_update_now (NetkWindow * window)
     update_wmhints (window);
     update_transient_for (window);      /* wintype needs this to be first */
     update_wintype (window);
+    update_icon_geometry (window);
     update_wm_state (window);
     update_state (window);      /* must come after the above, since they affect
                                  * our calculated state
                                  */
     update_workspace (window);  /* emits signals */
     update_actions (window);
-    
 
     get_icons (window);
 
