@@ -820,7 +820,7 @@ client_session_new (gint argc, gchar * argv[], gpointer data,
         if ((!had_display) && (dpy != NULL))
         {
             array[i++] = "--display";
-            array[i++] = (gchar *) gdk_display_get_name (dpy);
+            array[i++] = (gchar *)gdk_display_get_name(dpy);
         }
 
         array[i] = NULL;
@@ -831,3 +831,334 @@ client_session_new (gint argc, gchar * argv[], gpointer data,
 
     return session_client;
 }
+
+void client_session_free(SessionClient * session_client)
+{
+    /* session_client->data; */
+    /* session_client->session_connection; */
+    
+    if (session_client->current_state != SESSION_CLIENT_DISCONNECTED) {
+        disconnect(session_client);
+    }
+
+    g_free(session_client->client_id);
+    g_free(session_client->given_client_id);
+
+    g_free(session_client->current_directory);
+    g_free(session_client->program);
+    g_strfreev(session_client->clone_command);
+    g_strfreev(session_client->resign_command);
+    g_strfreev(session_client->restart_command);
+    g_strfreev(session_client->discard_command);
+    g_strfreev(session_client->shutdown_command);
+    g_free(session_client);
+}
+
+static gchar** safe_strvdup(gchar ** const source)
+{
+    int     count;
+    int     i;
+    gchar** result;
+    
+    if (source == NULL) {
+        return NULL;
+    }
+    
+    count = g_strv_length((gchar **)source);
+    result = g_new0(gchar*, count + 1);
+    
+    for (i = 0; i < count; ++i) {
+        result[i] = g_strdup(source[i]);
+    }
+
+    result[count] = NULL;
+    return result;
+}
+
+void client_session_set_save_phase_2_callback(SessionClient * session_client, save_phase_2_callback value)
+{
+    session_client->save_phase_2 = value;
+}
+
+void client_session_set_interact_callback(SessionClient * session_client, interact_callback value)
+{
+    session_client->interact = value;
+}
+
+void client_session_set_shutdown_cancelled_callback(SessionClient * session_client, shutdown_cancelled_callback value)
+{
+    session_client->shutdown_cancelled = value;
+}
+
+void client_session_set_save_complete_callback(SessionClient * session_client, save_complete_callback value)
+{
+    session_client->save_complete = value;
+}
+
+void client_session_set_die_callback(SessionClient * session_client, die_callback value)
+{
+    session_client->die = value;
+}
+
+void client_session_set_save_yourself_callback(SessionClient * session_client, save_yourself_callback value)
+{
+    session_client->save_yourself = value;
+}
+
+void client_session_set_data(SessionClient * session_client, gpointer value)
+{
+    session_client->data = value;
+}
+
+/*void client_session_set_session_connection(SessionClient * session_client, gpointer value)*/
+
+/*void client_session_set_current_state(SessionClient * session_client, SessionClientState value)*/
+
+void client_session_set_restart_style(SessionClient * session_client, SessionRestartStyle value)
+{
+    if (session_client->session_connection != NULL) {
+        g_error("SessionClient: changing \"restart_style\" while connected to session manager is not implemented.");
+        return;
+    }
+    
+    session_client->restart_style = value;
+}
+
+/* void client_session_set_interact_style(SessionClient * session_client, SessionInteractStyle value) */
+
+void client_session_set_priority(SessionClient * session_client, gchar value)
+{
+    if (session_client->session_connection != NULL) {
+        g_error("SessionClient: changing \"priority\" while connected to session manager is not implemented.");
+        return;
+    }
+    
+    session_client->priority = value;
+}
+
+void client_session_set_client_id(SessionClient * session_client, gchar const* value)
+{
+    if (session_client->session_connection != NULL) {
+        g_error("SessionClient: changing \"client_id\" while connected to session manager is not possible.");
+        return;
+    }
+    
+    if (session_client->client_id != value) {
+        g_free(session_client->client_id);
+        session_client->client_id = g_strdup(value);
+    }
+}
+
+/*void client_session_set_given_client_id(SessionClient * session_client, gchar const* value) */
+
+void client_session_set_current_directory(SessionClient * session_client, gchar const* value)
+{
+    if (session_client->session_connection != NULL) {
+        g_error("SessionClient: changing \"current_directory\" while connected to session manager is not implemented.");
+        return;
+    }
+    
+    if (session_client->current_directory != value) {
+        g_free(session_client->current_directory);
+        session_client->current_directory = g_strdup(value);
+    }
+}
+
+void client_session_set_program(SessionClient * session_client, gchar const* value)
+{
+    if (session_client->session_connection != NULL) {
+        g_error("SessionClient: changing \"program\" while connected to session manager is not implemented.");
+        return;
+    }
+    
+    if (session_client->program != value) {
+        g_free(session_client->program);
+        session_client->program = g_strdup(value);
+    }
+}
+
+void client_session_set_clone_command(SessionClient * session_client, gchar** const value)
+{
+    if (session_client->clone_command == value) {
+        return;
+    }
+    
+    if (session_client->clone_command != NULL) {
+        g_strfreev(session_client->clone_command);
+        session_client->clone_command = NULL;
+    }
+
+    if (value != NULL) {
+        session_client->clone_command = safe_strvdup(value);
+    }
+}
+
+void client_session_set_resign_command(SessionClient * session_client, gchar** const value)
+{
+    if (session_client->resign_command == value) {
+        return;
+    }
+    
+    if (session_client->resign_command != NULL) {
+        g_strfreev(session_client->resign_command);
+        session_client->resign_command = NULL;
+    }
+
+    if (value != NULL) {
+        session_client->resign_command = safe_strvdup(value);
+    }
+}
+
+void client_session_set_restart_command(SessionClient * session_client, gchar** const value)
+{
+    if (session_client->restart_command == value) {
+        return;
+    }
+    
+    if (session_client->restart_command != NULL) {
+        g_strfreev(session_client->restart_command);
+        session_client->restart_command = NULL;
+    }
+
+    if (value != NULL) {
+        session_client->restart_command = safe_strvdup(value);
+    }
+}
+
+void client_session_set_discard_command(SessionClient * session_client, gchar** const value)
+{
+    if (session_client->discard_command == value) {
+        return;
+    }
+    
+    if (session_client->discard_command != NULL) {
+        g_strfreev(session_client->discard_command);
+        session_client->discard_command = NULL;
+    }
+
+    if (value != NULL) {
+        session_client->discard_command = safe_strvdup(value);
+    }
+}
+
+void client_session_set_shutdown_command(SessionClient * session_client, gchar** const value)
+{
+    if (session_client->shutdown_command == value) {
+        return;
+    }
+    
+    if (session_client->shutdown_command != NULL) {
+        g_strfreev(session_client->shutdown_command);
+        session_client->shutdown_command = NULL;
+    }
+
+    if (value != NULL) {
+        session_client->shutdown_command = safe_strvdup(value);
+    }
+}
+
+save_phase_2_callback client_session_get_save_phase_2_callback(SessionClient * session_client)
+{
+    return session_client->save_phase_2;
+}
+
+interact_callback client_session_get_interact_callback(SessionClient * session_client)
+{
+    return session_client->interact;
+}
+
+shutdown_cancelled_callback client_session_get_shutdown_cancelled_callback(SessionClient * session_client)
+{
+    return session_client->shutdown_cancelled;
+}
+
+save_complete_callback client_session_get_save_complete_callback(SessionClient * session_client)
+{
+    return session_client->save_complete;
+}
+
+die_callback client_session_get_die_callback(SessionClient * session_client)
+{
+    return session_client->die;
+}
+
+save_yourself_callback client_session_get_save_yourself_callback(SessionClient * session_client)
+{
+    return session_client->save_yourself;
+}
+
+gpointer client_session_get_data(SessionClient * session_client)
+{
+    return session_client->data;
+}
+
+gpointer client_session_get_session_connection(SessionClient * session_client)
+{
+    return session_client->session_connection;
+}
+
+SessionClientState client_session_get_current_state(SessionClient * session_client)
+{
+    return session_client->current_state;
+}
+
+SessionRestartStyle client_session_get_restart_style(SessionClient * session_client)
+{
+    return session_client->restart_style;
+}
+
+SessionInteractStyle client_session_get_interact_style(SessionClient * session_client)
+{
+    return session_client->interact_style;
+}
+
+gchar client_session_get_priority(SessionClient * session_client)
+{
+    return session_client->priority;
+}
+
+gchar const* client_session_get_client_id(SessionClient * session_client)
+{
+    return session_client->client_id;
+}
+
+gchar const* client_session_get_given_client_id(SessionClient * session_client)
+{
+    return session_client->given_client_id;
+}
+
+gchar const* client_session_get_current_directory(SessionClient * session_client)
+{
+    return session_client->current_directory;
+}
+
+gchar const* client_session_get_program(SessionClient * session_client)
+{
+    return session_client->program;
+}
+
+G_CONST_RETURN gchar* G_CONST_RETURN * client_session_get_clone_command(SessionClient * session_client)
+{
+    return (const gchar * const *) session_client->clone_command;
+}
+
+G_CONST_RETURN gchar* G_CONST_RETURN * client_session_get_resign_command(SessionClient * session_client)
+{
+    return (const gchar * const *) session_client->resign_command;
+}
+
+G_CONST_RETURN gchar* G_CONST_RETURN * client_session_get_restart_command(SessionClient * session_client)
+{
+    return (const gchar * const *) session_client->restart_command;
+}
+
+G_CONST_RETURN gchar* G_CONST_RETURN * client_session_get_discard_command(SessionClient * session_client)
+{
+    return (const gchar * const *) session_client->discard_command;
+}
+
+G_CONST_RETURN gchar* G_CONST_RETURN * client_session_get_shutdown_command(SessionClient * session_client)
+{
+    return (const gchar * const *) session_client->shutdown_command;
+}
+
